#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "wav.h"

static int read_chars(FILE *in, char *s, int n)
{
int t,ch;

  for (t=0; t<n; t++)
  {
    ch=getc(in);
    if (ch==EOF) return -1;
    s[t]=ch;
  }
  s[t]=0;

  return 0;
}

/*
static int write_int(FILE *out, int n)
{
  putc((n&255),out);
  putc(((n>>8)&255),out);
  putc(((n>>16)&255),out);
  putc(((n>>24)&255),out);

  return 0;
}

static int write_short(FILE *out, int n)
{

  putc((n&255),out);
  putc(((n>>8)&255),out);

  return 0;
}
*/

static unsigned int read_int(FILE *in)
{
unsigned int t;

  t=getc(in);
  t=(getc(in)<<8)|t;
  t=(getc(in)<<16)|t;
  t=(getc(in)<<24)|t;

  return t;
}

static unsigned short int read_short(FILE *in)
{
unsigned short int t;

  t=getc(in);
  t=(getc(in)<<8)|t;

  return t;
}

int parse_header(FILE *in)
{
int length;
char fourcc[5];
char riff_type[5];

  read_chars(in,fourcc,4);
  if (strncmp(fourcc,"RIFF",4)!=0)
  {
    printf("This file is not a RIFF/WAV\n");
    return -1;
  }

  length=read_int(in);
  read_chars(in,riff_type,4);

  printf("RIFF Header\n");
  printf("----------------------------\n");
  printf("          Length: %d\n",length);
  printf("            Type: %s\n",riff_type);
  printf("----------------------------\n");

  return 0;
}

int parse_fmt_chunk(FILE *in, struct fmt_chunk_t *fmt_chunk)
{
char chunk_type[5];
int length;

  read_chars(in,chunk_type,4);
  length=read_int(in);
  fmt_chunk->format_type=read_short(in);
  fmt_chunk->channel_numbers=read_short(in);
  fmt_chunk->sample_rate=read_int(in);
  fmt_chunk->bytes_per_second=read_int(in);
  fmt_chunk->bytes_per_sample=read_short(in);
  fmt_chunk->bits_per_sample=read_short(in);

  printf("FMT Chunk\n");
  printf("----------------------------\n");
  printf("      Chunk Type: %s\n",chunk_type);
  printf("          Length: %d\n",length);
  printf("     Format Type: ");
  if (fmt_chunk->format_type==0)
  { printf("Mono\n"); }
    else
  if (fmt_chunk->format_type==1)
  { printf("Stereo\n"); }
    else
  { printf("unknown\n"); }

  printf(" Channel Numbers: %d\n",fmt_chunk->channel_numbers);
  printf("     Sample Rate: %d\n",fmt_chunk->sample_rate);
  printf("Bytes Per Second: %d\n",fmt_chunk->bytes_per_second);
  printf("Bytes Per Sample: ");
  if (fmt_chunk->bytes_per_sample==1)
  { printf("8 bit mono (%d)\n",fmt_chunk->bytes_per_sample); }
    else
  if (fmt_chunk->bytes_per_sample==2)
  { printf("8 bit stereo or 16 bit mono (%d)\n",fmt_chunk->bytes_per_sample); }
    else
  if (fmt_chunk->bytes_per_sample==4)
  { printf("16 bit stereo (%d)\n",fmt_chunk->bytes_per_sample); }
    else
  { printf("?\n"); }

  printf(" Bits Per Sample: %d\n",fmt_chunk->bits_per_sample);
  printf("----------------------------\n");

  return 0;
}

int parse_data_chunk(FILE *in)
{
int length;
char riff_type[5];

  read_chars(in,riff_type,4);
  length=read_int(in);

  printf("Data Header\n");
  printf("----------------------------\n");
  printf("          Length: %d\n",length);
  printf("            Type: %s\n",riff_type);
  printf("----------------------------\n");

  if (strncmp(riff_type,"data",4)!=0)
  {
    return 0;
  }

  return length;
}


