#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "kohn_gif.h"
#include "wav.h"

#define DEFAULT_WIDTH 640
#define DEFAULT_HEIGHT 480

void draw_line(unsigned char *picture,int x0,int y0,int x1,int y1,int width,int height)
{
double x,y;
double dx,dy;

// printf("(%d,%d)-(%d,%d)\n",x0,y0,x1,y1);

  dx=x1-x0;
  dy=y1-y0;

  if (x0==x1 && y0==y1)
  {
    picture[y0*width+x0]=1;
  }
    else
  if (abs(dx)>abs(dy))
  {
    if (x0>x1) { x=x0; x0=x1; x1=x; }
    dy=dy/abs(dx);
    y=y0;

    for (x=x0; x<x1; x=x+1)
    {
      picture[(int)y*width+(int)x]=1;
      y=y+dy;
    }
  }
    else
  {
    if (y0>y1) { y=y0; y0=y1; y1=y; }
    dx=dx/abs(dy);
    x=x0;

    for (y=y0; y<y1; y=y+1)
    {
      picture[(int)y*width+(int)x]=1;
      x=x+dx;
    }
  }
}

int main(int argc, char *argv[])
{
FILE *in=0,*out=0;
struct gif_info_t *output_gif;
struct fmt_chunk_t fmt_chunk;
double duration=0.01,start=0;
unsigned char *picture;
int samples=0,start_sample;
int channel=0;
int width=DEFAULT_WIDTH,height=DEFAULT_HEIGHT;
int length,bytes_per_sample;
int x,y,x0,y0;
int t,r,ch;
signed char s8;  /* incase this is run on embedded CPU, call this signed */
short int s16;

  if (argc<3)
  {
    printf("wav2gif - Copyright 2007 by Michael Kohn\n");
    printf("Version: 2007-10-26\n\n");
    printf("Usage: wav2gif [ optional options ] <input_filename> <output_filename>\n");
    printf("      -s <start point in seconds [0.0]>\n");
    printf("      -d <duration in seconds [%f]>\n",duration);
    printf("      -m <number of samples (overrides -d)>\n");
    printf("      -w <image width [%d]>\n",DEFAULT_WIDTH);
    printf("      -h <image height [%d]>\n",DEFAULT_HEIGHT);
    printf("\n");
  }

  for (t=1; t<argc; t++)
  {
    if (strcmp(argv[t],"-s")==0)
    { start=atof(argv[++t]); }
      else
    if (strcmp(argv[t],"-d")==0)
    { duration=atof(argv[++t]); }
      else
    if (strcmp(argv[t],"-m")==0)
    { samples=atoi(argv[++t]); }
      else
    if (strcmp(argv[t],"-c")==0)
    { channel=atoi(argv[++t]); }
      else
    if (strcmp(argv[t],"-w")==0)
    { width=atoi(argv[++t]); }
      else
    if (strcmp(argv[t],"-h")==0)
    { height=atoi(argv[++t]); }
      else
    if (argv[t][0]=='-')
    {
      printf("Uknown option %s\n",argv[t]);
      exit(1);
    }
      else
    {
      if (in==0)
      {
        in=fopen(argv[t],"rb");
        if (in==0)
        {
          printf("Cannot open input file %s\n",argv[t]);
          exit(1);
        }
      }
        else
      if (out==0)
      {
        out=fopen(argv[t],"wb");
        if (out==0)
        {
          printf("Cannot open output file %s\n",argv[t]);
          fclose(in);
          exit(1);
        }
      }
    }
  }

  if (in==0)
  {
    printf("No input file\n");
    exit(1);
  }

  if (out==0)
  {
    printf("No output file\n");
    fclose(in);
    exit(1);
  }

  parse_header(in);
  parse_fmt_chunk(in,&fmt_chunk);
  length=parse_data_chunk(in);

  if (length<=0)
  {
    printf("Cannot read this WAV file\n");
  }
    else
  if (channel>=fmt_chunk.channel_numbers)
  {
    printf("Selected channel doesn't exist\n");
  }
    else
  {
    if (samples==0)
    { samples=(double)fmt_chunk.sample_rate*duration; }
    start_sample=(double)fmt_chunk.sample_rate*start;
    bytes_per_sample=fmt_chunk.channel_numbers*(fmt_chunk.bits_per_sample/8);

    printf("\n");
    printf(" Starting Sample: %d\n",start_sample);
    printf("   Total Samples: %d\n",samples);
    printf("Bytes Per Sample: %d\n",bytes_per_sample);
    printf("\n");

    picture=(unsigned char *)malloc(width*height);
    memset(picture,0,width*height);

    for (t=0; t<width; t++) picture[(height/2)*width+t]=2;

    fseek(in,start_sample*bytes_per_sample,SEEK_CUR);

    x0=-1; y0=-1;

    for (t=0; t<samples; t++)
    {
      // draw_line(picture,100,200,200,100,width,height);
      // break;

      x=(int)(((double)t/(double)samples)*(double)width);

      if (fmt_chunk.bits_per_sample==8)
      {
        ch=getc(in);
        if (ch==EOF) break;
        s8=(char)ch;

        y=(height/2)+(s8*(height/2)/128);

        for (r=1; r<fmt_chunk.channel_numbers; r++) getc(in);
      }
        else
      if (fmt_chunk.bits_per_sample==16)
      {
        ch=getc(in);
// printf("ch=%d\n",ch);
        if (ch==EOF) break;
        s16=ch;
        ch=getc(in);
// printf("ch=%d\n",ch);
        if (ch==EOF) break;
        ch=s16|(ch<<8);
        s16=(short int)ch;

// printf("%d\n",s16);
        y=(height/2)+(s16*(height/2)/32768);
        for (r=1; r<fmt_chunk.channel_numbers; r++) { getc(in); getc(in); }
      }
        else
      { y=height/2; }

      if (x>=0 && x<width && y>=0 && y<height)
      {
        if (x0==-1) { x0=x; y0=y; }
        draw_line(picture,x0,y0,x,y,width,height);
        // picture[y*width+x]=1;
        x0=x;
        y0=y;
      }
    }

    output_gif=kgif_alloc_compress();
    output_gif->fp=out;
    output_gif->width=width;
    output_gif->height=height;
    output_gif->colors=4;
    output_gif->color_map[0]=0xffffff;
    output_gif->color_map[1]=0xff0000;
    output_gif->color_map[2]=0x0000ff;

    kgif_write_header(output_gif);
    kgif_compress_block(output_gif,picture,width*height);
    kgif_write_footer(output_gif);
    kgif_info_free(output_gif);

    free(picture);
  }



  fclose(in);
  fclose(out);

  return 0;
}


